@@ -7,7 +7,7 @@ try:
     from spython.main import get_client
     from spython.utils import check_install
     USE_DOCKER = False
-    client = get_client()
+    client = get_client(quiet=True)
     if not check_install():
         raise ImportError("Singularity not available")
 except ImportError:
@@ -17,21 +17,26 @@ except ImportError:
 
 logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.INFO)
 
-# where everything lives
-tmpdir = tempfile.TemporaryDirectory(prefix=os.getcwd() + "/tmp")
+# Create a temporary directory to hold all the writable parts of the configuration + any
+# data stored in the databases. We put this in a special user directory so that all the files
+# are self-conained
+tmpdir = os.makedirs(os.getcwd() + "/tmp/", exist_ok=True)
+dbdir = tempfile.TemporaryDirectory(prefix=os.getcwd() + "/tmp/")
 
 class Service:
     singularity_container: str
     docker_container: Tuple[str, str]
 
     @classmethod
-    def pull(cls):
+    def pull(cls, singularity_filename: str):
         if USE_DOCKER:
             logging.info(f"Pulling contaner: {cls.docker_container}")
             client.images.pull(cls.docker_container[0], tag=cls.docker_container[1])
-        else:
+        elif not os.path.exists(singularity_filename):
             logging.info(f"Pulling contaner: {cls.singularity_container}")
             client.pull(image=cls.singularity_container)
+        else:
+            logging.info(f"Using existing image: {singularity_filename}")
 
     @classmethod
     def run(cls):
@@ -43,12 +48,12 @@ class MongoDB(Service):
     docker_container = ("mongo", "latest")
 
     @classmethod
-    def run(cls):
-        cls.pull()
+    def run(cls, image_file: str = "mongo:latest.sif"):
+        cls.pull(image_file)
         if USE_DOCKER:
             return client.containers.run(cls.docker_container[0], detach=True, auto_remove=False, name="data-engineering-mongodb")
         else:
-            inst = client.instance.instance(image=self.singularity_container, options=[f"-B {tmpdir.name}:/data/db", f"-B {tmpdir.name}:/tmp"])
+            inst = client.instance.instance(image=image_file, options=[f"-B {dbdir.name}:/data/db", f"-B {dbdir.name}:/tmp"])
             def _run_mongo():
                 client.execute(image=f"instance://{inst.name}", command=["mongod"])
             t = threading.Thread(target=_run_mongo)
@@ -61,13 +66,13 @@ class TimescaleDB(Service):
     docker_container = ("timescale/timescaledb", "latest-pg14")
 
     @classmethod
-    def run(cls):
-        cls.pull()
+    def run(cls, image_file: str = "timescaledb:latest-pg14.sif"):
+        cls.pull(image_file)
         if USE_DOCKER:
             return client.containers.run(cls.docker_container[0], detach=True, auto_remove=False, name="data-engineering-timescaledb")
         else:
             os.environ["POSTGRES_PASSWORD"] = "password"
-            inst = client.instance.instance(image=cls.singularity_container, options=[f"-B {tmpdir.name}:/var/lib/postgresql/data", f"-B {tmpdir.name}:/var/run/postgresql", f"--env=POSTGRES_PASSWORD"])
+            inst = client.instance.instance(image=image_file, options=[f"-B {dbdir.name}:/var/lib/postgresql/data", f"-B {dbdir.name}:/var/run/postgresql", f"--env=POSTGRES_PASSWORD"])
             def _run_timescale():
                 client.execute(image=f"instance://{inst.name}", command=["/usr/local/bin/docker-entrypoint.sh", "postgres"])
             t = threading.Thread(target=_run_timescale)
